@@ -24,6 +24,8 @@ from .version_framework import version
 from http.server import BaseHTTPRequestHandler, HTTPServer
 from .types import TypesActions
 from .HttpResponse import HttpResponse
+import redis
+from Crypto.Hash import SHA256
 
 if os.environ.get('KAFKA', None):
     from .ms_base import KafkaBase as BaseClass
@@ -77,6 +79,9 @@ class Microservices(BaseClass, ABC):
         Clase que contiene la logica del microservicio.
     """
 
+    # Instancia de redis
+    redisclient = None
+
     def __init__(self, opt):
         # Validar que se pase un objeto de configuracion correcto
         if not isinstance(opt, Options):
@@ -127,6 +132,10 @@ class Microservices(BaseClass, ABC):
         # llamar el constructor padre
         BaseClass.__init__(self, topico, opt.Hosts, topico)
 
+        # Inicializar el redis
+        if os.environ.get('REDISVALIDATE', None) is not None:
+            self.initRedis()
+
         if opt.isNats:
             # iniciar la aplicacion
             """
@@ -142,6 +151,33 @@ class Microservices(BaseClass, ABC):
             finally:
                 self.loop.close()
 
+    def initRedis(self):
+        """
+            Metodo para inicializar el redis
+        """
+        if os.environ.get('REDISVALIDATE', None) is None:
+            self.logs.warn('No se encuentra activado la validacion por redis')
+        else:
+            port = os.environ.get('REDIS_PORT', 6379)
+            self.redisclient = redis.StrictRedis(
+                host=os.environ.get('REDISVALIDATE'), port=int(port), db=0)
+            self.logs.info('Se encuentra activado la validacion por redis')
+
+    def hasProcessMessage(self, data):
+        """
+            Metodo para validar que una operacion ya fue relizada.
+        """
+        if os.environ.get('REDISVALIDATE', None) is None:
+            return False
+        else:
+            h = SHA256.new()
+            h.update(bytes(data, 'utf-8'))
+            response_redis = self.redisclient.execute_command(
+                'GETSET', h.hexdigest(), data)
+            if response_redis is None:
+                return False
+            return True
+
     def initilizeTracer(self):
         config = Config(
             config={
@@ -321,6 +357,30 @@ class Microservices(BaseClass, ABC):
             span.finish()
             return
 
+        # Validacion de redis
+        if self.hasProcessMessage('{}_{}_{}'.format(
+            self.__TOPIC__,
+            meta.get('id_transaction', ''),
+            json.dumps(msg.get('data', {})
+                       )
+        )):
+            span = None
+
+            if rootSpan is None:
+                span = self.___Tracer.start_span(
+                    operation_name='replication')
+            else:
+                span = self.___Tracer.start_span(
+                    operation_name='replication', child_of=rootSpan)
+
+            if os.environ.get('DISABLED_LOGS_VALIDATE', None) is None:
+                self.logs.error('Se recibio una operacion que ya se realizo')
+
+            span.set_tag(ext_tags.ERROR, 'hasProcessMessage')
+
+            span.finish()
+            return True
+
         # Solo actualizar los datos si e el servicio original
         if self.__TOPIC__ == msg['metadata']['callback']:
             if not msg['metadata'].get('data_inicial', None) is None:
